﻿using System;
using System.Data.OleDb;
using System.IO;
using System.Net;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.Class1Listener
{
    class Resource : HierarchyItem, IResource
    {
        public Resource(Request request, int id, string name, DateTime created, DateTime modified, string path)
            : base(request, id, name, created, modified, path)
        {
        }

        #region IResource Members
        
        public long ContentLength
        {
            get
            {
                long length = 0;

                OleDbConnection conn = new OleDbConnection(connStr);
                OleDbDataReader reader = null;
                conn.Open();

                try
                {
                    OleDbCommand cmd = conn.CreateCommand();
                    cmd.CommandText = "SELECT Content FROM Repository WHERE ID = ?";
                    cmd.Parameters.Add("@ID", OleDbType.Integer).Value = ID;

                    reader = cmd.ExecuteReader();
                    reader.Read();
                    if (!reader.IsDBNull(reader.GetOrdinal("Content")))
                        length = reader.GetBytes(reader.GetOrdinal("Content"), 0, null, 0, 0);
                }
                finally
                {
                    if (reader != null) reader.Close();
                    conn.Close();
                }

                return length;
            }
        }

        public string ContentType
        {
            get
            {
                string contentType = "";

                OleDbConnection conn = new OleDbConnection(connStr);
                OleDbDataReader reader = null;
                conn.Open();

                try
                {
                    OleDbCommand cmd = conn.CreateCommand();
                    cmd.CommandText = "SELECT ContentType FROM Repository WHERE ID = ?";
                    cmd.Parameters.Add("@ID", OleDbType.Integer).Value = ID;
                    reader = cmd.ExecuteReader();
                    reader.Read();
                    if (!reader.IsDBNull(reader.GetOrdinal("ContentType")))
                        contentType = reader.GetString(reader.GetOrdinal("ContentType"));
                }
                finally
                {
                    if (reader != null) reader.Close();
                    conn.Close();
                }

                if (contentType == "")
                    contentType =
                        MimeType.GetMimeType(System.IO.Path.GetExtension(this.name))
                        ?? "application/octet-stream";
                return contentType;
            }
        }

        public WebDAVResponse SaveFromStream(Stream stream, string contentType)
        {
            WebDAVResponse resp;

            OleDbConnection conn = new OleDbConnection(connStr);
            conn.Open();

            try
            {
                OleDbCommand cmd = conn.CreateCommand();
                cmd.CommandText = "UPDATE Repository SET"
                    + " Content = ?,"
                    + " Modified = ?,"
                    + " ContentType = ?"
                    + " WHERE ID = ?";
                
                if (stream != Stream.Null)
                {
                    int bufSize = 1024000;
                    int byteCount;
                    byte[] buf = new byte[bufSize];
                    MemoryStream contentStream = new MemoryStream();

                    while ((byteCount = stream.Read(buf, 0, bufSize)) > 0)
                    {
                        contentStream.Write(buf, 0, byteCount);
                    }

                    byte[] content = new byte[contentStream.Length];

                    contentStream.Position = 0;
                    contentStream.Read(content, 0, (int)contentStream.Length);

                    cmd.Parameters.Add("@Content", OleDbType.Binary).Value = content;

                    resp = new OkResponse();
                }
                else
                {
                    cmd.Parameters.Add("@Content", OleDbType.Binary).Value = DBNull.Value;
                    resp = new NoContentResponse();
                }

                cmd.Parameters.Add("@Modified", OleDbType.Date).Value = DateTime.UtcNow;
                if (contentType != null)
                    cmd.Parameters.Add("@ContentType", OleDbType.WChar).Value = contentType;
                else
                    cmd.Parameters.Add("@ContentType", OleDbType.WChar).Value = DBNull.Value;
                cmd.Parameters.Add("@ID", OleDbType.Integer).Value = ID;

                cmd.ExecuteNonQuery();
            }
            finally
            {
                conn.Close();
            }

            return resp;
        }

        public WebDAVResponse WriteToStream(Stream output, long startIndex, long count)
        {
            OleDbConnection conn = new OleDbConnection(connStr);
            OleDbDataReader reader = null;
            conn.Open();

            try
            {
                OleDbCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT Content FROM Repository WHERE ID = ?";
                cmd.Parameters.Add("@ID", OleDbType.Integer).Value = ID;
                reader = cmd.ExecuteReader();
                reader.Read();
                long length = reader.GetBytes(reader.GetOrdinal("Content"), 0, null, 0, 0);
                if (length > 0)
                {
                    byte[] buf = new byte[count];
                    reader.GetBytes(reader.GetOrdinal("Content"), startIndex, buf, 0, (int)count);
                    try
                    {
                        output.Write(buf, 0, (int)count);
                    }
                    catch (HttpListenerException)
                    { // client closed connection
                        // 1. ErrorCode=1229. An operation was attempted on a nonexistent network connection.
                        // 2. ErrorCode=64. The specified network name is no longer available.
                        return new OkResponse();
                    }
                }
            }
            finally
            {
                if (reader != null) reader.Close();
                conn.Close();
            }

            return new OkResponse();
        }

        #endregion // IResource Members

        #region IHierarchyItem Members Override

        public override WebDAVResponse MoveTo(IFolder folder, string destName)
        {
            Folder destFolder = (Folder) folder;
            IHierarchyItem destItem = destFolder.FindChild(destName);

            if (destItem != null)
            {
                destItem.Delete();
                base.MoveTo(folder, destName);
                return new NoContentResponse();
            }
            else
            {
                base.MoveTo(folder, destName);
                return new CreatedResponse();
            }
        }

        #endregion // IHierarchyItem Members
    }
}
