﻿using System;
using System.Configuration;
using System.Data.OleDb;
using ITHit.WebDAV.Logger;
using ITHit.WebDAV.Server;

namespace WebDAVServer.Class1Listener
{
    class WDEngine : Engine
    {
        private string connStr = ConfigurationManager.ConnectionStrings["OleDbConnectionString"].ConnectionString;
        
        #region Engine Members        

        public override IHierarchyItem GetHierarchyItem(string path)
        {
            FileLogger.WriteMessage("GetHierarchyItem call: path = " + path, LogLevel.Debug);

            int itemID = 0;
            ItemType itemType = 0;
            string itemName = "";
            DateTime itemCreated = DateTime.MinValue;
            DateTime itemModified = DateTime.MinValue;

            OleDbConnection conn = new OleDbConnection(connStr);
            OleDbDataReader reader = null;
            conn.Open();

            try
            {
                OleDbCommand cmd = conn.CreateCommand();
                if (path == "/")
                {
                    // get root folder
                    cmd.CommandText =
                        "SELECT ID, ItemType, Name, Created, Modified"
                        + " FROM Repository"
                        + " WHERE Parent = 0";
                    reader = cmd.ExecuteReader();
                    if (reader.Read())
                    {
                        itemID = reader.GetInt32(reader.GetOrdinal("ID"));
                        itemType = (ItemType)reader.GetByte(reader.GetOrdinal("ItemType"));
                        itemName = reader.GetString(reader.GetOrdinal("Name"));
                        itemCreated = reader.GetDateTime(reader.GetOrdinal("Created"));
                        itemModified = reader.GetDateTime(reader.GetOrdinal("Modified"));
                    }
                }
                else
                {
                    // find root
                    cmd.CommandText = "SELECT ID FROM Repository WHERE Parent = 0";
                    int id = (int)cmd.ExecuteScalar();

                    string[] names = path.Split('/');
                    int last = names.Length - 1;
                    while (last > 0 && names[last] == "") last--;
                    // search for item by path
                    cmd.CommandText =
                        "SELECT ID FROM Repository"
                        + " WHERE Name = ?"
                        + " AND Parent = ?";
                    cmd.Parameters.Add("@Name", OleDbType.WChar);
                    cmd.Parameters.Add("@Parent", OleDbType.Integer);
                    for (int i = 0; i < last; i++)
                        if (names[i] != "")
                        {
                            cmd.Parameters["@Name"].Value = names[i];
                            cmd.Parameters["@Parent"].Value = id;
                            id = (int)cmd.ExecuteScalar();
                        }
                    // get item properties
                    cmd.CommandText =
                        "SELECT ID, ItemType, Name, Created, Modified"
                        + " FROM Repository"
                        + " WHERE Name = ?"
                        + " AND Parent = ?";
                    cmd.Parameters.Clear();
                    cmd.Parameters.Add("@Name", OleDbType.WChar).Value = names[last];
                    cmd.Parameters.Add("@Parent", OleDbType.Integer).Value = id;
                    reader = cmd.ExecuteReader();
                    if (reader.Read())
                    {
                        itemID = reader.GetInt32(reader.GetOrdinal("ID"));
                        itemType = (ItemType)reader.GetByte(reader.GetOrdinal("ItemType"));
                        itemName = reader.GetString(reader.GetOrdinal("Name"));
                        itemCreated = reader.GetDateTime(reader.GetOrdinal("Created"));
                        itemModified = reader.GetDateTime(reader.GetOrdinal("Modified"));
                    }
                }
            }
            finally
            {
                if (reader != null) reader.Close();
                conn.Close();
            }

            switch (itemType)
            {
                case ItemType.Resource:
                    return new Resource(this.Request, itemID, itemName, itemCreated, itemModified, path);
                case ItemType.Folder:
                    return new Folder(this.Request, itemID, itemName, itemCreated, itemModified, path);
                default:
                    return null;
            }
        }
       
        #endregion // Engine Members
    }
}
