using System;
using System.Data;
using System.Data.SqlClient;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.DeltaV
{
    public class LockNull : HierarchyItem, ILockNull
    {
        public LockNull(WDEngine engine, Guid id, string name, string path, DateTime created, DateTime modified)
            : base(engine, id, name, path, created, modified)
        {
        }

        #region ILockNull Members

        public WebDAVResponse ConvertToResource()
        {
            if (!ClientHasToken)
                return new LockedResponse();

            CreateItemFromLockNull(ItemId, ItemType.Resource);
            return new CreatedResponse();
        }

        public WebDAVResponse ConvertToFolder()
        {
            if (!ClientHasToken)
                return new LockedResponse();

            CreateItemFromLockNull(ItemId, ItemType.Folder);

            return new CreatedResponse();
        }

        #endregion  // ILockNull Members

        #region HierarchyItem members overriding

        public override WebDAVResponse CopyTo(IFolder folder, string destName, bool deep)
        {
            return new NotAllowedResponse();
        }

        public override WebDAVResponse MoveTo(IFolder folder, string destName)
        {
            return new NotAllowedResponse();
        }

        public override WebDAVResponse Delete()
        {
            return new NotAllowedResponse();
        }

        public override WebDAVResponse UpdateProperties(Property[] setProps, Property[] delProps)
        {
            return new NotAllowedResponse();
        }

        #endregion  // HierarchyItem members overriding

        private void CreateItemFromLockNull(Guid lockNullID, ItemType itemType)
        {
            SqlCommand cmd = fEngine.CreateNewCommand();

            cmd.CommandText = "UPDATE Item SET ItemType = @ItemType WHERE ItemId = @ItemId";
            cmd.Parameters.Add("@ItemType", SqlDbType.Int).Value = itemType;
            cmd.Parameters.Add("@ItemId", SqlDbType.UniqueIdentifier).Value = lockNullID;
            cmd.ExecuteNonQuery();

            UpdateModified();
        }
    }
}