using System.Configuration;
using System.IO;
using ITHit.WebDAV.Logger;
using ITHit.WebDAV.Server;

namespace WebDAVServer.DeltaV
{
    /// <summary>
    /// This handler processes requests to folders returning custom HTML page.
    /// </summary>
    class MyCustomGetHandler : IMethodHandler
    {
        private IMethodHandler originalHandler;

        public IMethodHandler OriginalHandler
        {
            set { originalHandler = value; }
        }

        public void ProcessRequest(Request request, IResponse response, IHierarchyItem item)
        {
            if (item is IFolder)
            {
                using (TextReader reader = File.OpenText(ConfigurationManager.AppSettings["CustomHandlerPage"]))
                {
                    string html = reader.ReadToEnd();
                    html = html.Replace("_webDavServerRoot_", request.ApplicationPath);
                    response.ContentLength = html.Length; // always set content length in custom handler when using HttpListener, otherwise the content will not be sent.
                    using (StreamWriter writer = new StreamWriter(response.OutputStream))
                    {
                        writer.Write(html);
                    }
                    FileLogger.WriteMessage(html, LogLevel.Debug);
                }
            }
            else
            {
                originalHandler.ProcessRequest(request, response, item);
            }
        }
    }
}
