using System;
using System.Configuration;
using System.Data.SqlClient;
using System.IO;
using ITHit.WebDAV.Server.DeltaV;
using System.ServiceProcess;
using ITHit.WebDAV.Logger;

namespace WebDAVServer.DeltaV
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            if (!AppConfigHasErrors())
                if (IsServiceMode)
                {
                    ServiceBase.Run(new DeltaVService());
                }
                else
                {
                    (new DeltaVService(true)).ThreadProc();
                }
        }

        private static bool IsServiceMode
        {
            get
            {
                return !Environment.UserInteractive;
            }
        }

        private static bool AppConfigHasErrors()
        {
            bool errorOccured = false;
            string errorMessage = string.Empty;

            try
            {
                string autoput = ConfigurationManager.AppSettings["AutoPutUnderVersionControl"];
                if (autoput != null)
                    Convert.ToBoolean(autoput);
                else throw new Exception();
            }
            catch
            {
                errorMessage = "AutoPutUnderVersionControl section is missing or invalid! Should contain \"true/false\" value only.";
                errorOccured = true;
            }

            try
            {
                string delWhenRepl = ConfigurationManager.AppSettings["DeleteWhenReplace"];
                if (delWhenRepl != null)
                    Convert.ToBoolean(delWhenRepl);
                else throw new Exception();
            }
            catch
            {
                errorMessage = "DeleteWhenReplace section is missing or invalid! Should contain \"true/false\" value only.";
                errorOccured = true;
            }

            SqlConnection conn = null;

            try
            {
                string connStr = ConfigurationManager.AppSettings["SqlConnectionString"];
                if (connStr != null)
                {
                    conn = new SqlConnection(connStr);
                    conn.Open();
                }
                else throw new Exception();
            }
            catch
            {
                errorMessage = "SqlConnectionString section is missing or invalid!";
                errorOccured = true;
            }
            finally
            {
                if (conn != null) conn.Close();
            }

            try
            {
                string repPath = ConfigurationManager.AppSettings["RepositoryPath"];
                if (repPath == null || !Directory.Exists(repPath))
                    throw new Exception();
            }
            catch
            {
                errorMessage = "RepositoryPath section is missing or invalid!";
                errorOccured = true;
            }

            try
            {
                string autoVerMode = ConfigurationManager.AppSettings["AutoVersionMode"];
                if (autoVerMode != null)
                {
                    AutoVersion autoVer = (AutoVersion) Enum.Parse(typeof (AutoVersion), autoVerMode);
                    switch(autoVer)
                    {
                        case AutoVersion.NoAutoVersioning:
                        case AutoVersion.CheckOutCheckIn:
                        case AutoVersion.CheckOutUnlockedCheckIn:
                        case AutoVersion.CheckOut:
                        case AutoVersion.LockedCheckOut:
                            break;
                        default:
                            throw new Exception();
                    }
                }
                else throw new Exception();
            }
            catch
            {
                errorMessage = "AutoVersionMode section is missing or invalid!\nPossible values are: (NoAutoVersioning | CheckOutCheckIn | CheckOutUnlockedCheckIn | CheckOut | LockedCheckOut).";
                errorOccured = true;
            }

            try
            {
                string uriPrefix = ConfigurationManager.AppSettings["ListenerPrefix"];
                if (uriPrefix == null)
                    throw new Exception();
            }
            catch
            {
                errorMessage = "ListenerPrefix section is missing or invalid!";
                errorOccured = true;
            }

            try
            {
                string authType = ConfigurationManager.AppSettings["AuthenticationType"];
                if (authType == null)
                    throw new Exception();
                switch (authType.ToLower())
                {
                    case "none":
                    case "basic":
                    case "digest":
                    case "ntlm":
                    case "kerberos":
                        break;
                    default:
                        throw new Exception();
                }
            }
            catch
            {
                errorMessage = "AuthenticationType section is missing or invalid!\nPossible values are: (None | Basic | Digest | Ntlm | Kerberos).";
                errorOccured = true;
            }
            
            if (errorOccured)
            {
                if (IsServiceMode)
                {
                    errorMessage = "Could not start service because of errors in config file\n     " + errorMessage;
                    FileLogger.WriteMessage(errorMessage, LogLevel.Fatal);
                }
                else
                {
                    Console.WriteLine(errorMessage);
                    Console.WriteLine("\nPress any key...");
                    Console.ReadKey();
                }
            }

            return errorOccured;
        }
    }
}