using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.DeltaV;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.DeltaV
{
    class VersionHistory : IHierarchyItem, IHistory
    {
        private WDEngine fEngine;
        private Guid fItemId;
        private string fPath;

        public VersionHistory(WDEngine engine, Guid itemId, string path)
        {
            this.fEngine = engine;
            this.fItemId = itemId;
            this.fPath = path;
        }

        #region IHistory Members

        public IVersion CurrentVersion
        {
            get
            {
                return GetCurrentOrRootVersion(Versions.Current)[0];
            }
        }

        public IVersion[] VersionSet
        {
             get
             {
                 return GetCurrentOrRootVersion(Versions.All);
             }
        }

        public IVersion RootVersion
        {
             get
             {
                 return GetCurrentOrRootVersion(Versions.Root)[0];
             }
        }

        #endregion // IHistory Members

        #region IHierarchyItem Members
      
        public WebDAVResponse CopyTo(IFolder folder, string destName, bool deep)
        {
            return new NotAllowedResponse();
        }

        public WebDAVResponse MoveTo(IFolder folder, string destName)
        {
            return new NotAllowedResponse();
        }

        public WebDAVResponse Delete()
        {
            return new NotAllowedResponse();
        }

        public string Name
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public IFolder Parent
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public DateTime Created
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public DateTime Modified
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public WebDAVResponse GetProperties(ref Property[] props)
        {
            return new NotAllowedResponse();
        }

        public WebDAVResponse GetPropertyNames(ref Property[] props)
        {
            return new NotAllowedResponse();
        }

        public WebDAVResponse UpdateProperties(Property[] setProps, Property[] delProps)
        {
            return new NotAllowedResponse();
        }

        public string Path
        {
            get { return fPath; }
        }

        #endregion // IHierarchyItem Members

        private IVersion[] GetCurrentOrRootVersion(Versions versions)
        {
            List<IVersion> version = new List<IVersion>();

            SqlCommand cmd = fEngine.CreateNewCommand();
            cmd.CommandText = "SELECT VersionId, VersionNumber, Name, Created"
                              + " FROM Version WHERE ItemId = @ItemId"
                              + ((versions == Versions.All) ? "" : " AND VersionNumber ="
                              + " (SELECT " + ((versions == Versions.Current) ? "MAX" : "MIN") + "(VersionNumber)"
                              + " FROM Version"
                              + " WHERE ItemId = @ItemId)");
            cmd.Parameters.Add("@ItemId", SqlDbType.UniqueIdentifier).Value = this.fItemId;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.HasRows)
                    while (reader.Read())
                    {
                        Guid versionId = reader.GetGuid(reader.GetOrdinal("VersionId"));
                        int versionNumber = reader.GetInt32(reader.GetOrdinal("VersionNumber"));
                        string versionFileName = reader.GetString(reader.GetOrdinal("Name"));
                        DateTime versionCreated = reader.GetDateTime(reader.GetOrdinal("Created"));

                        version.Add(new Version(fEngine, versionId, fItemId, versionNumber, versionFileName,
                                                Version.CreateVersionPath(fPath.Remove(fPath.IndexOf('?')),
                                                                          versionNumber), versionCreated));
                    }
            }
            return version.ToArray();
        }
    }

    internal enum Versions
    {
        All = 0,
        Root = 1,
        Current = 2
    }
}
