using System.Configuration;
using System.IO;
using System.Web;
using ITHit.WebDAV.Server;

namespace WebDAVServer.NtfsStorage
{
	public class WDEngine : Engine
	{
        private string repositoryPath = ConfigurationManager.AppSettings["RepositoryPath"].TrimEnd('\\');

        #region Engine Members Override

        public override IHierarchyItem GetHierarchyItem(string path)
        {
            int ind = path.IndexOf('?');
            if (ind > 0)
                path = path.Remove(ind);
            string fullPath = repositoryPath + '\\' + path.Replace('/', '\\');

            DirectoryInfo directory = new DirectoryInfo(fullPath);
            if (directory.Exists)
                return new Folder(directory, this);
            else
            {
                FileInfo file = new FileInfo(fullPath);
                if (file.Exists)
                    return new Resource(file, this);
            }
            return null;
        }
        
		// By default WebDAV engine searches for License.lic in the folder where your .dll or .exe file resides.
		// To change this behavior Engine.License is overridden
		public override string License
		{
			get
			{
			    string license;
			    using (TextReader reader = File.OpenText(HttpContext.Current.Request.PhysicalApplicationPath + "License.lic"))
			    {
			        license = reader.ReadToEnd();
			    }
			    return license;
			}
        }	    

	    #endregion // Engine Members Override
    }
}
