using System;
using System.Web;
using System.Text;
using System.Security.Principal;

namespace WebDAVServer.SqlStorage
{
	public class BasicAuthenticationModule : IHttpModule
	{
        #region IHttpModule Members

        public void Init(HttpApplication application)
		{
			application.AuthenticateRequest += new EventHandler(this.App_OnAuthenticateRequest);
			application.EndRequest += new EventHandler(App_OnEndRequest);
		}

        public void Dispose()
		{
        }

        #endregion // IHttpModule Members

        #region Non-public Members

        private void App_OnAuthenticateRequest(object source, EventArgs eventArgs)
		{
			HttpApplication app = (HttpApplication) source;
			
			string auth = app.Request.Headers["Authorization"];
			if ( (auth==null) || (auth.Substring(0, 5).ToLower()!="basic") )
			{ // no basic authorization header found
				Unauthorized();
				return; 
			}
			
			// decode username and password
			string base64Credentials = auth.Substring(6);
			byte[] bytesCredentials = Convert.FromBase64String(base64Credentials);
			string[] credentials = new UTF8Encoding().GetString(bytesCredentials).Split(':');
			string userName = credentials[0];
			string password = credentials[1];

			// Windows Vista sends user name in the form DOMAIN\User
			int delimiterIndex = userName.IndexOf('\\');
			if(delimiterIndex!=-1)
                userName = userName.Remove(0, delimiterIndex+1);

			if(Authenticate(userName, password)) 
			{ // authenticated succesefully
				app.Context.User = new GenericPrincipal(new GenericIdentity(userName), null);
			}
			else
			{ // invalid credentials
				Unauthorized();
			}
		}

		protected virtual bool Authenticate(string userName, string password)
		{
			return (userName=="User1") && (password=="pwd");
		}

		private static void Unauthorized()
		{
			HttpResponse response = HttpContext.Current.Response;
			response.StatusCode = 401;
			response.StatusDescription = "Unauthorized";
			response.Write("401 Unauthorized");
		}

		private static void App_OnEndRequest(object source, EventArgs eventArgs)
		{
			HttpApplication app = (HttpApplication) source;
			if (app.Response.StatusCode == 401)
			{ // show login dialog
				app.Response.AppendHeader("WWW-Authenticate", "Basic Realm=\"My WebDAV Server\"");
			}
        }

        #endregion // Non-public Members
    }
}
