using System;
using System.Data;
using System.Data.SqlClient;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.SqlStorage
{
	public class LockNull : HierarchyItem, ILockNull
	{
        public LockNull(int id, string name, string path, DateTime created, DateTime modified, WDEngine engine)
            :base(id, name, path, created, modified, engine)
        {
        }

        #region HierarchyItem Members Override

        public override WebDAVResponse CopyTo(IFolder folder, string destName, bool deep)
		{
			return new NotAllowedResponse();
		}

		public override WebDAVResponse MoveTo(IFolder folder, string destName)
		{
			return new NotAllowedResponse();
		}

		public override WebDAVResponse Delete()
		{
			 return new NotAllowedResponse();
		}

		public override WebDAVResponse UpdateProperties(Property[] setProps, Property[] delProps)
		{
			return new NotAllowedResponse();
        }

        #endregion // HierarchyItem Members Override

        #region ILockNull Members

        public WebDAVResponse ConvertToResource()
		{
			if(!ClientHasToken)
				return new LockedResponse();

            using (SqlConnection conn = new SqlConnection(connStr))
            {
                SqlTransaction trans = null;
                conn.Open();
                try
                {
                    trans = conn.BeginTransaction(IsolationLevel.ReadUncommitted);
                    CreateItemFromLockNull(trans, ID, ItemType.Resource);
                    trans.Commit();
                }
                catch
                {
                    if (trans != null) trans.Rollback();
                    throw;
                }
            }
            return new CreatedResponse();
		}

		public WebDAVResponse ConvertToFolder()
		{
			if(!ClientHasToken)
				return new LockedResponse();

		    using (SqlConnection conn = new SqlConnection(connStr))
		    {
		        SqlTransaction trans = null;
		        conn.Open();
		        try
		        {
		            trans = conn.BeginTransaction(IsolationLevel.ReadUncommitted);
		            CreateItemFromLockNull(trans, ID, ItemType.Folder);
		            trans.Commit();
		        }
		        catch
		        {
		            if(trans != null) trans.Rollback();
		            throw;
		        }
		    }

		    return new CreatedResponse();
        }

        #endregion // ILockNull Members

        private void CreateItemFromLockNull(SqlTransaction trans, int lockNullID, ItemType itemType)
        {
            SqlCommand cmd = trans.Connection.CreateCommand();
            cmd.Transaction = trans;

            // set item type
            cmd.CommandText = "UPDATE Repository SET ItemType = @ItemType WHERE ID = @ID";
            cmd.Parameters.Add("@ItemType", SqlDbType.TinyInt).Value = itemType;
            cmd.Parameters.Add("@ID", SqlDbType.Int).Value = lockNullID;
            cmd.ExecuteNonQuery();

            UpdateModified(trans);
        }
    }
}
